@@ -66,15 +66,21 @@ namespace {
 
 
 
+SmartctlParser::SmartctlParser()
+		: disk_type_(StorageAttribute::DiskAny)
+{ }
+
 
 
 // Parse full "smartctl -a" output
-bool SmartctlParser::parse_full(const std::string& full)
+bool SmartctlParser::parse_full(const std::string& full, StorageAttribute::DiskType disk_type)
 {
 	this->clear();  // clear previous data
 
 	this->set_data_full(full);
 
+	disk_type_ = disk_type;
+
 
 	// -------------------- Fix the output so it doesn't interfere with proper parsing
 
@@ -201,21 +207,20 @@ bool SmartctlParser::parse_full(const std::string& full)
 	while (section_start_pos != std::string::npos
 			&& (section_start_pos = s.find("===", section_start_pos)) != std::string::npos) {
 
-		tmp_pos = s.find("\n", section_start_pos);  // works with \r\n too.
+		tmp_pos = s.find("\n", section_start_pos);  // works with \r\n too. This may be npos if nothing follows the header.
 
 		// trim is needed to remove potential \r in the end
 		std::string section_header = hz::string_trim_copy(s.substr(section_start_pos,
 				(tmp_pos == std::string::npos ? tmp_pos : (tmp_pos - section_start_pos)) ));
 
-		if (tmp_pos != std::string::npos)
-			++tmp_pos;  // set to start of the next section
-
-		section_end_pos = s.find("===", tmp_pos);  // start of the next section
-		std::string section_body_str = hz::string_trim_copy(s.substr(tmp_pos,
-				(section_end_pos == std::string::npos ? section_end_pos : section_end_pos - tmp_pos)));
-
+		std::string section_body_str;
+		if (tmp_pos != std::string::npos) {
+			section_end_pos = s.find("===", tmp_pos);  // start of the next section
+			section_body_str = hz::string_trim_copy(s.substr(tmp_pos,
+					(section_end_pos == std::string::npos ? section_end_pos : section_end_pos - tmp_pos)));
+		}
 		status = parse_section(section_header, section_body_str) || status;
-		section_start_pos = section_end_pos;
+		section_start_pos = (tmp_pos == std::string::npos ? std::string::npos : section_end_pos);
 	}
 
 	if (!status) {
@@ -408,52 +413,47 @@ bool SmartctlParser::parse_section_info_property(StorageProperty& p)
 	}
 
 
-	if (app_pcre_match("/Model Family/mi", p.reported_name)) {
+	if (app_pcre_match("/^Model Family$/mi", p.reported_name)) {
 		p.set_name(p.reported_name, "model_family", "Model Family");
 		p.value_type = StorageProperty::value_type_string;
 		p.value_string = p.reported_value;
 
-	} else if (app_pcre_match("/Device Model/mi", p.reported_name)) {
+	} else if (app_pcre_match("/^(?:Device Model|Device|Product)$/mi", p.reported_name)) {  // "Device" and "Product" are from scsi/usb
 		p.set_name(p.reported_name, "device_model", "Device Model");
 		p.value_type = StorageProperty::value_type_string;
 		p.value_string = p.reported_value;
 
-	} else if (app_pcre_match("/Serial Number/mi", p.reported_name)) {
+	} else if (app_pcre_match("/^Vendor$/mi", p.reported_name)) {  // From scsi/usb
+		p.set_name(p.reported_name, "vendor", "Vendor");
+		p.value_type = StorageProperty::value_type_string;
+		p.value_string = p.reported_value;
+
+	} else if (app_pcre_match("/^Revision$/mi", p.reported_name)) {  // From scsi/usb
+		p.set_name(p.reported_name, "revision", "Revision");
+		p.value_type = StorageProperty::value_type_string;
+		p.value_string = p.reported_value;
+
+	} else if (app_pcre_match("/^Device type$/mi", p.reported_name)) {  // From scsi/usb
+		p.set_name(p.reported_name, "device_type", "Device Type");
+		p.value_type = StorageProperty::value_type_string;
+		p.value_string = p.reported_value;
+
+	} else if (app_pcre_match("/^Serial Number$/mi", p.reported_name)) {
 		p.set_name(p.reported_name, "serial_number", "Serial Number");
 		p.value_type = StorageProperty::value_type_string;
 		p.value_string = p.reported_value;
 
-	} else if (app_pcre_match("/Firmware Version/mi", p.reported_name)) {
-		p.set_name(p.reported_name, "firmware_version", "Firmware Version");
-		p.value_type = StorageProperty::value_type_string;
-		p.value_string = p.reported_value;
-
-	} else if (app_pcre_match("/Sector Sizes/mi", p.reported_name)) {
-		p.set_name(p.reported_name, "sector_sizes", "Sector Sizes");
-		p.value_type = StorageProperty::value_type_string;  // prints 2 values (phys/logical, if they're different)
-		p.value_string = p.reported_value;
-
-	} else if (app_pcre_match("/Sector Size/mi", p.reported_name)) {
-		p.set_name(p.reported_name, "sector_size", "Sector Size");
-		p.value_type = StorageProperty::value_type_string;  // prints a single value (if it's not 512)
-		p.value_string = p.reported_value;
-
-	} else if (app_pcre_match("/LU WWN Device Id/mi", p.reported_name)) {
+	} else if (app_pcre_match("/^LU WWN Device Id$/mi", p.reported_name)) {
 		p.set_name(p.reported_name, "wwn_id", "World Wide Name");
 		p.value_type = StorageProperty::value_type_string;
 		p.value_string = p.reported_value;
 
-	} else if (app_pcre_match("/ATA Standard is/mi", p.reported_name)) {
-		p.set_name(p.reported_name, "ata_standard", "ATA Standard");
+	} else if (app_pcre_match("/^Firmware Version$/mi", p.reported_name)) {
+		p.set_name(p.reported_name, "firmware_version", "Firmware Version");
 		p.value_type = StorageProperty::value_type_string;
 		p.value_string = p.reported_value;
 
-	} else if (app_pcre_match("/Local Time is/mi", p.reported_name)) {
-		p.set_name(p.reported_name, "scan_time", "Scanned on");
-		p.value_type = StorageProperty::value_type_string;
-		p.value_string = p.reported_value;
-
-	} else if (app_pcre_match("/User Capacity/mi", p.reported_name)) {
+	} else if (app_pcre_match("/^User Capacity$/mi", p.reported_name)) {
 		p.set_name(p.reported_name, "capacity", "Capacity");
 		p.value_type = StorageProperty::value_type_integer;
 		uint64_t v = 0;
@@ -463,19 +463,52 @@ bool SmartctlParser::parse_section_info_property(StorageProperty& p)
 			p.value_integer = v;
 		}
 
-	} else if (app_pcre_match("/ATA Version is/mi", p.reported_name)) {
-		p.set_name(p.reported_name, "ata_version", "ATA Version");
-		p.value_type = StorageProperty::value_type_integer;
-		int64_t v = 0;
-		if (hz::string_is_numeric(p.reported_value, v, true))  // strict mode
-			p.value_integer = v;
+	} else if (app_pcre_match("/^Sector Sizes$/mi", p.reported_name)) {
+		p.set_name(p.reported_name, "sector_sizes", "Sector Sizes");
+		p.value_type = StorageProperty::value_type_string;  // prints 2 values (phys/logical, if they're different)
+		p.value_string = p.reported_value;
 
-	} else if (app_pcre_match("/Device is/mi", p.reported_name)) {
+	} else if (app_pcre_match("/^Sector Size$/mi", p.reported_name)) {
+		p.set_name(p.reported_name, "sector_size", "Sector Size");
+		p.value_type = StorageProperty::value_type_string;  // prints a single value (if it's not 512)
+		p.value_string = p.reported_value;
+
+	} else if (app_pcre_match("/^Logical block size$/mi", p.reported_name)) {  // from scsi/usb
+		p.set_name(p.reported_name, "logical_block_size", "Logical Block Size");
+		p.value_type = StorageProperty::value_type_string;  // "512 bytes"
+		p.value_string = p.reported_value;
+
+	} else if (app_pcre_match("/^Rotation Rate$/mi", p.reported_name)) {
+		p.set_name(p.reported_name, "rotation_rate", "Rotation Rate");
+		p.value_type = StorageProperty::value_type_string;
+		p.value_string = p.reported_value;
+
+	} else if (app_pcre_match("/^Device is$/mi", p.reported_name)) {
 		p.set_name(p.reported_name, "in_smartctl_db", "In Smartctl Database");
 		p.value_type = StorageProperty::value_type_bool;
 		p.value_bool = (!app_pcre_match("/Not in /mi", p.reported_value));
 
-	} else if (app_pcre_match("/SMART support is/mi", p.reported_name)) {
+	} else if (app_pcre_match("/^ATA Version is$/mi", p.reported_name)) {
+		p.set_name(p.reported_name, "ata_version", "ATA Version");
+		p.value_type = StorageProperty::value_type_string;
+		p.value_string = p.reported_value;
+
+	} else if (app_pcre_match("/^ATA Standard is$/mi", p.reported_name)) {
+		p.set_name(p.reported_name, "ata_standard", "ATA Standard");
+		p.value_type = StorageProperty::value_type_string;
+		p.value_string = p.reported_value;
+
+	} else if (app_pcre_match("/^SATA Version is$/mi", p.reported_name)) {
+		p.set_name(p.reported_name, "sata_version", "SATA Version");
+		p.value_type = StorageProperty::value_type_string;
+		p.value_string = p.reported_value;
+
+	} else if (app_pcre_match("/^Local Time is$/mi", p.reported_name)) {
+		p.set_name(p.reported_name, "scan_time", "Scanned on");
+		p.value_type = StorageProperty::value_type_string;
+		p.value_string = p.reported_value;
+
+	} else if (app_pcre_match("/^SMART support is$/mi", p.reported_name)) {
 		// There are two different properties with this name - supported and enabled.
 		// Don't put complete messages here - they change across smartctl versions.
 
@@ -499,15 +532,17 @@ bool SmartctlParser::parse_section_info_property(StorageProperty& p)
 			p.value_type = StorageProperty::value_type_bool;
 			p.value_bool = false;
 
-		// this should be last - when ambiguous state is detected, usually smartctl
+		// this should be the last - when ambiguous state is detected, usually smartctl
 		// retries with other methods and prints one of the above.
 		} else if (app_pcre_match("/Ambiguous/mi", p.reported_value)) {
 			p.set_name(p.reported_name, "smart_supported", "SMART Supported");
 			p.value_type = StorageProperty::value_type_bool;
 			p.value_bool = true;  // let's be optimistic - just hope that it doesn't hurt.
-
 		}
 
+	} else if (app_pcre_match("/^scsiMode/mi", p.reported_name)) {  // these are some debug warnings from smartctl on usb flash drives
+		p.show_in_ui = false;
+
 	} else {
 		debug_out_warn("app", DBG_FUNC_MSG << "Unknown property \"" << p.reported_name << "\"\n");
 		// this is not an error, just unknown attribute. treat it as string.
@@ -574,15 +609,18 @@ bool SmartctlParser::parse_section_data(const std::string& body)
 			status = parse_section_data_subsection_attributes(sub) || status;
 
 		} else if (app_pcre_match("/SMART Error Log Version/mi", sub)
-				|| app_pcre_match("/Warning: device does not support Error Logging/mi", sub)) {
+				|| app_pcre_match("/Warning: device does not support Error Logging/mi", sub)
+				|| app_pcre_match("/SMART Error Log not supported/mi", sub)) {
 			status = parse_section_data_subsection_error_log(sub) || status;
 
 		} else if (app_pcre_match("/SMART Self-test log/mi", sub)
-				|| app_pcre_match("/Warning: device does not support Self Test Logging/mi", sub)) {
+				|| app_pcre_match("/Warning: device does not support Self Test Logging/mi", sub)
+				|| app_pcre_match("/SMART Self-test Log not supported/mi", sub)) {
 			status = parse_section_data_subsection_selftest_log(sub) || status;
 
 		} else if (app_pcre_match("/SMART Selective self-test log data structure/mi", sub)
-				|| app_pcre_match("/Device does not support Selective Self Tests/Logging/mi", sub)) {
+				|| app_pcre_match("/Device does not support Selective Self Tests\\/Logging/mi", sub)
+				|| app_pcre_match("/Selective Self-tests\\/Logging not supported/mi", sub)) {
 			status = parse_section_data_subsection_selective_selftest_log(sub) || status;
 
 		} else {
@@ -1298,7 +1336,7 @@ bool SmartctlParser::parse_section_data_subsection_error_log(const std::string&
 
 	// Error log support
 	{
-		pcrecpp::RE re = app_pcre_re("/^(Warning: device does not support Error Logging)$/mi");
+		pcrecpp::RE re = app_pcre_re("/^(Warning: device does not support Error Logging)|(SMART Error Log not supported)$/mi");
 
 		if (re.PartialMatch(sub)) {
 			StorageProperty p(pt);
@@ -1441,7 +1479,7 @@ bool SmartctlParser::parse_section_data_subsection_selftest_log(const std::strin
 
 	// Self-test log support
 	{
-		pcrecpp::RE re = app_pcre_re("/^(Warning: device does not support Self Test Logging)$/mi");
+		pcrecpp::RE re = app_pcre_re("/^(Warning: device does not support Self Test Logging)|(SMART Self-test Log not supported)$/mi");
 
 		if (re.PartialMatch(sub)) {
 			StorageProperty p(pt);
@@ -1621,7 +1659,7 @@ bool SmartctlParser::parse_section_data_subsection_selective_selftest_log(const
 // Yes, there's no place for this in the Parser, but whatever...
 void SmartctlParser::add_property(StorageProperty p)
 {
-	storage_property_autoset_description(p);
+	storage_property_autoset_description(p, disk_type_);
 	storage_property_autoset_warning(p);
 	storage_property_autoset_warning_descr(p);  // append warning to description
 
