@@ -10,22 +10,16 @@
 /// @{
 
 #include <map>
-#include <gtkmm/button.h>
-#include <gtkmm/entry.h>
-#include <gtkmm/checkbutton.h>
-#include <gtkmm/treeview.h>
-#include <gtkmm/liststore.h>
-#include <gtkmm/filechooserdialog.h>
-#include <gtkmm/stock.h>
-#include <gtkmm/accelgroup.h>
-#include <gtkmm/messagedialog.h>
-#include <gdk/gdkkeysyms.h>  // GDK_Escape
+#include <gtkmm.h>
+#include <gdk/gdk.h>  // GDK_KEY_Escape
 
 #include "hz/fs_path.h"
 #include "hz/string_sprintf.h"
+#include "hz/scoped_ptr.h"
 #include "rconfig/rconfig_mini.h"
 #include "applib/storage_settings.h"
 #include "applib/app_gtkmm_utils.h"
+#include "gsc_main_window.h"
 
 #include "gsc_preferences_window.h"
 
@@ -177,9 +171,9 @@ class GscPreferencesDeviceOptionsTreeView : public Gtk::TreeView {
 				Gtk::TreeModel::Row row = *iter;
 				std::string dev = row.get_value(col_device_real);
 				if (!dev.empty()) {
-					std::string par = row.get_value(col_type_real);
-					if (!par.empty()) {
-						dev += "::" + par;
+					std::string type = row.get_value(col_type_real);
+					if (!type.empty()) {
+						dev += "::" + type;
 					}
 					if (devmap.find(dev) == devmap.end()) {
 						devmap[dev] = row.get_value(col_parameters);
@@ -248,6 +242,17 @@ GscPreferencesWindow::GscPreferencesWindow(BaseObjectType* gtkcobj, const app_ui
 	APP_UI_RES_AUTO_CONNECT(window_reset_all_button, clicked);
 
 
+	Glib::ustring smartctl_binary_tooltip = "A path to smartctl binary. If the path is not absolute, the binary will be looked for in user's PATH.";
+#if defined CONFIG_KERNEL_FAMILY_WINDOWS
+	smartctl_binary_tooltip += Glib::ustring("\n") + "Note: smartctl.exe shows a console during execution, while smartctl-nc.exe (default) doesn't (nc means no-console).";
+#endif
+	if (Gtk::Label* smartctl_binary_label = lookup_widget<Gtk::Label*>("smartctl_binary_label")) {
+		app_gtkmm_set_widget_tooltip(*smartctl_binary_label, smartctl_binary_tooltip);
+	}
+	if (Gtk::Entry* smartctl_binary_entry = lookup_widget<Gtk::Entry*>("smartctl_binary_entry")) {
+		app_gtkmm_set_widget_tooltip(*smartctl_binary_entry, smartctl_binary_tooltip);
+	}
+
 	Gtk::Button* smartctl_binary_browse_button = 0;
 	APP_UI_RES_AUTO_CONNECT(smartctl_binary_browse_button, clicked);
 
@@ -262,11 +267,11 @@ GscPreferencesWindow::GscPreferencesWindow(BaseObjectType* gtkcobj, const app_ui
 	Gtk::Entry* device_options_device_entry = 0;
 	APP_UI_RES_AUTO_CONNECT(device_options_device_entry, changed);
 
-	Glib::ustring device_options_tooltip = "Device name";
+	Glib::ustring device_options_tooltip = "A device name to match";
 #if defined CONFIG_KERNEL_FAMILY_WINDOWS
-	device_options_tooltip = "Device name (for example, use \"pd0\" for the first physical drive)";
+	device_options_tooltip = "A device name to match (for example, use \"pd0\" for the first physical drive)";
 #elif defined CONFIG_KERNEL_LINUX
-	device_options_tooltip = "Device name (for example, /dev/sda or /dev/twa0)";
+	device_options_tooltip = "A device name to match (for example, /dev/sda or /dev/twa0)";
 #endif
 	if (Gtk::Label* device_options_device_label = lookup_widget<Gtk::Label*>("device_options_device_label")) {
 		app_gtkmm_set_widget_tooltip(*device_options_device_label, device_options_tooltip);
@@ -279,18 +284,6 @@ GscPreferencesWindow::GscPreferencesWindow(BaseObjectType* gtkcobj, const app_ui
 	Gtk::Entry* device_options_type_entry = 0;
 	APP_UI_RES_AUTO_CONNECT(device_options_type_entry, changed);
 
-	Glib::ustring device_type_tooltip = "Match only this type of device (as specified to -d smartctl parameter)";
-#if defined CONFIG_KERNEL_LINUX
-	device_type_tooltip = "Match only this type of device (as specified to -d smartctl parameter). Leave empty for all types. This can be used to specify a drive behind a RAID device, e.g. \"3ware,2\".";
-#endif
-	if (Gtk::Label* device_options_type_label = lookup_widget<Gtk::Label*>("device_options_type_label")) {
-		app_gtkmm_set_widget_tooltip(*device_options_type_label, device_type_tooltip);
-	}
-	if (device_options_type_entry) {
-		app_gtkmm_set_widget_tooltip(*device_options_type_entry, device_type_tooltip);
-	}
-
-
 	Gtk::Entry* device_options_parameter_entry = 0;
 	APP_UI_RES_AUTO_CONNECT(device_options_parameter_entry, changed);
 
@@ -299,7 +292,7 @@ GscPreferencesWindow::GscPreferencesWindow(BaseObjectType* gtkcobj, const app_ui
 
 	Glib::RefPtr<Gtk::AccelGroup> accel_group = this->get_accel_group();
 	if (window_cancel_button) {
-		window_cancel_button->add_accelerator("clicked", accel_group, GDK_Escape,
+		window_cancel_button->add_accelerator("clicked", accel_group, GDK_KEY_Escape,
 				Gdk::ModifierType(0), Gtk::AccelFlags(0));
 	}
 
@@ -328,6 +321,13 @@ GscPreferencesWindow::GscPreferencesWindow(BaseObjectType* gtkcobj, const app_ui
 
 
 
+void GscPreferencesWindow::set_main_window(GscMainWindow* window)
+{
+	main_window_ = window;
+}
+
+
+
 
 void GscPreferencesWindow::update_device_widgets(const std::string& device, const std::string& type, const std::string& params)
 {
@@ -496,7 +496,6 @@ void GscPreferencesWindow::export_config()
 	device_option_map_t devmap = device_options_treeview->get_device_map();
 	std::string devmap_str = app_serialize_device_option_map(devmap);
 	prefs_config_set("system/smartctl_device_options", devmap_str);
-
 }
 
 
@@ -518,7 +517,32 @@ void GscPreferencesWindow::on_window_cancel_button_clicked()
 
 void GscPreferencesWindow::on_window_ok_button_clicked()
 {
+	// Check if device map contains drives with empty device names or parameters.
+	device_option_map_t devmap = device_options_treeview->get_device_map();
+	bool contains_empty = false;
+	for (device_option_map_t::const_iterator iter = devmap.begin(); iter != devmap.end(); ++iter) {
+		if (iter->first.empty() || iter->second.empty()) {
+			contains_empty = true;
+		}
+	}
+
+	if (contains_empty) {
+		Gtk::MessageDialog dialog(*this,
+				"You have specified an empty Parameters field for one or more entries"
+				" in Per-Drive Smartctl Parameters section. Such entries will be discarded.\n"
+				"\nDo you want to continue?",
+				true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO, true);
+		if (dialog.run() != Gtk::RESPONSE_YES) {
+			return;
+		}
+	}
+
 	export_config();
+
+	if (main_window_) {
+		main_window_->show_prefs_updated_message();
+	}
+
 	destroy(this);
 }
 
@@ -548,6 +572,20 @@ void GscPreferencesWindow::on_smartctl_binary_browse_button_clicked()
 	if (!entry)
 		return;
 
+	hz::FsPath path(entry->get_text());
+
+	int result = 0;
+
+#if GTK_CHECK_VERSION(3, 20, 0)
+	hz::scoped_ptr<GtkFileChooserNative> dialog(gtk_file_chooser_native_new(
+			"Choose Smartctl Binary...", this->gobj(), GTK_FILE_CHOOSER_ACTION_OPEN, NULL, NULL), g_object_unref);
+
+	if (path.is_absolute())
+		gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog.get()), path.c_str());
+
+	result = gtk_native_dialog_run(GTK_NATIVE_DIALOG(dialog.get()));
+
+#else
 	Gtk::FileChooserDialog dialog(*this, "Choose Smartctl Binary...",
 			Gtk::FILE_CHOOSER_ACTION_OPEN);
 
@@ -556,18 +594,24 @@ void GscPreferencesWindow::on_smartctl_binary_browse_button_clicked()
 	dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_ACCEPT);
 
 	// Note: This works on absolute paths only (otherwise it's gtk warning).
-	hz::FsPath p(entry->get_text());
-	if (p.is_absolute())
-		dialog.set_filename(p.str());  // change to its dir and select it if exists.
+	if (path.is_absolute())
+		dialog.set_filename(path.str());  // change to its dir and select it if exists.
 
 	// Show the dialog and wait for a user response
-	int result = dialog.run();  // the main cycle blocks here
+	result = dialog.run();  // the main cycle blocks here
+#endif
 
 	// Handle the response
 	switch (result) {
 		case Gtk::RESPONSE_ACCEPT:
 		{
-			entry->set_text(std::string(dialog.get_filename()));
+			Glib::ustring file;
+#if GTK_CHECK_VERSION(3, 20, 0)
+			file = app_ustring_from_gchar(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog.get())));
+#else
+			file = dialog.get_filename();  // in fs encoding
+#endif
+			entry->set_text(file);
 			break;
 		}
 
