@@ -12,7 +12,7 @@
 #ifndef GSC_PREFERENCES_WINDOW_H
 #define GSC_PREFERENCES_WINDOW_H
 
-#include <gtkmm/window.h>
+#include <gtkmm.h>
 
 #include "applib/app_ui_res_utils.h"
 
@@ -20,6 +20,8 @@
 
 class GscPreferencesDeviceOptionsTreeView;  // defined in cpp file
 
+class GscMainWindow;  // declared in gsc_main_window.h
+
 
 
 /// The Preferences window.
@@ -38,6 +40,9 @@ class GscPreferencesWindow : public AppUIResWidget<GscPreferencesWindow, true> {
 		virtual ~GscPreferencesWindow()
 		{ }
 
+		/// Set main window so that we can manipulate it
+		void set_main_window(GscMainWindow* window);
+
 
 		/// Update the device widgets (per-device parameters), used by GscPreferencesDeviceOptionsTreeView.
 		void update_device_widgets(const std::string& device, const std::string& type, const std::string& params);
@@ -97,6 +102,8 @@ class GscPreferencesWindow : public AppUIResWidget<GscPreferencesWindow, true> {
 
 	private:
 
+		GscMainWindow* main_window_;  ///< Main window that called us.
+
 		GscPreferencesDeviceOptionsTreeView* device_options_treeview;  ///< Device options tree view
 
 };
